@@ -5,8 +5,9 @@ use warnings;
 use GraphQL::Schema;
 use GraphQL::Debug qw(_debug);
 use Lingua::EN::Inflect::Number qw(to_S);
+use Carp qw(confess);
 
-our $VERSION = "0.08";
+our $VERSION = "0.09";
 use constant DEBUG => $ENV{GRAPHQL_DEBUG};
 
 my %GRAPHQL_TYPE2SQLS = (
@@ -101,11 +102,13 @@ my %TYPEMAP = (
     } @{ $GRAPHQL_TYPE2SQLS{$gql_type} }
   } keys %GRAPHQL_TYPE2SQLS),
   enum => sub {
-    my $info = shift;
+    my ($source, $column, $info) = @_;
     my $extra = $info->{extra};
     return {
       kind => 'enum',
-      name => _dbicsource2pretty($extra->{custom_type_name}),
+      name => _dbicsource2pretty(
+        $extra->{custom_type_name} || "${source}_$column"
+      ),
       values => { map { _trim_name($_) => { value => $_ } } @{ $extra->{list} } },
     }
   },
@@ -114,6 +117,7 @@ my %TYPE2SCALAR = map { ($_ => 1) } qw(ID String Int Float Boolean);
 
 sub _dbicsource2pretty {
   my ($source) = @_;
+  confess "_dbicsource2pretty given undef" if !defined $source;
   $source = eval { $source->source_name } || $source;
   $source =~ s#.*::##;
   $source = to_S $source;
@@ -250,7 +254,7 @@ sub to_graphql {
       DEBUG and _debug("schema_dbic2graphql($name.col)", $column, $info);
       my $rawtype = $TYPEMAP{ lc $info->{data_type} };
       if ( 'CODE' eq ref $rawtype ) {
-        my $col_spec = $rawtype->($info);
+        my $col_spec = $rawtype->($name, $column, $info);
         push @ast, $col_spec unless $seentype{$col_spec->{name}};
         $rawtype = $col_spec->{name};
         $seentype{$col_spec->{name}} = 1;
